"""
init
"""

default_app_config: str = "fleetfinder.apps.FleetFinderConfig"

github_url = "https://github.com/ppfeufer/aa-fleetfinder"

__title__ = "Fleet Finder"
__version__ = "0.1.0-alpha.9"

__verbose_name__ = "Fleet Finder for Alliance Auth"
__user_agent_name__ = "Fleet-Finder-for-Alliance-Auth"

__user_agent__ = f"{__verbose_name__} v{__version__} {github_url}"
