# Generated by Django 3.2.9 on 2021-11-15 12:31

# Django
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("fleetfinder", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="fleet",
            name="motd",
            field=models.TextField(blank=True, default=""),
        ),
    ]
